@@ -25,6 +25,7 @@ import secrets
 import requests
 import time
 import threading
+import importlib
 
 
 def get_current_version():
@@ -46,7 +47,11 @@ def check_for_updates():
 
         if latest_version != current_version and current_version != "unknown":
             print(f"\n✨ Update available: {current_version} → {latest_version}")
-            print("   Run: pip install --upgrade omnara\n")
+            print("   Run: pip install --upgrade omnara")
+            print(
+                "   Please keep omnara up-to-date for best compatibility with Claude Code"
+            )
+            print("   New versions often include important bug fixes\n")
     except Exception:
         pass
 
@@ -279,20 +284,18 @@ def authenticate_via_browser(auth_url="https://omnara.com"):
     state = secrets.token_urlsafe(32)
 
     # Start local server to receive the callback
-    server = AuthHTTPServer(("localhost", 0), AuthCallbackHandler)
+    server = AuthHTTPServer(("127.0.0.1", 0), AuthCallbackHandler)
     server.state = state
     server.api_key = None
     port = server.server_port
 
     # Construct the auth URL
     auth_base = auth_url.rstrip("/")
-    callback_url = f"http://localhost:{port}"
-    auth_url = f"{auth_base}/cli-auth?callback={urllib.parse.quote(callback_url)}&state={urllib.parse.quote(state)}"
+    auth_url = f"{auth_base}/cli-auth?port={port}&state={urllib.parse.quote(state)}"
 
     print("\nOpening browser for authentication...")
-    print("If your browser doesn't open automatically, please click this link:")
+    print("If your browser doesn't open automatically, visit this link:")
     print(f"\n  {auth_url}\n")
-    print("Waiting for authentication...")
 
     # Run server in a thread
     server_thread = threading.Thread(target=server.serve_forever)
@@ -305,13 +308,62 @@ def authenticate_via_browser(auth_url="https://omnara.com"):
     except Exception:
         pass
 
-    # Wait for authentication (with timeout)
+    print("After signing in to Omnara:")
+    print("  • Local CLI: Click 'Authenticate Local CLI' button in your browser")
+    print("  • Remote/SSH: Copy the API key and paste below")
+
+    # Simple blocking input with timeout check in background
+    print(
+        "\nPaste API key here (or wait for browser authentication): ",
+        end="",
+        flush=True,
+    )
+
+    import subprocess
+
+    api_key = None
     start_time = time.time()
-    while not server.api_key and (time.time() - start_time) < 300:
+    timeout = 300
+
+    # Create a subprocess to read input that we can ACTUALLY KILL
+    proc = subprocess.Popen(
+        [sys.executable, "-c", "import sys; print(sys.stdin.readline().strip())"],
+        stdout=subprocess.PIPE,
+        stderr=subprocess.PIPE,
+        stdin=sys.stdin,
+        text=True,
+    )
+
+    while time.time() - start_time < timeout:
+        # Check if browser authenticated
+        if server.api_key:
+            print("\n✓ Authentication successful!")
+            api_key = server.api_key
+            proc.kill()  # KILL the subprocess - this actually works!
+            break
+
+        # Check if user pasted token
+        if proc.poll() is not None:  # Process finished
+            if proc.stdout:
+                output = proc.stdout.read().strip()
+                if output:
+                    print("✓ Token received!")
+                    api_key = output
+                    break
+
         time.sleep(0.1)
 
-    # If we got the API key, wait a bit for the browser to process the redirect
-    if server.api_key:
+    # Make sure subprocess is dead
+    try:
+        proc.kill()
+    except ProcessLookupError:
+        pass  # Process already dead
+
+    if not api_key:
+        print("\n✗ Authentication timed out")
+
+    # If we got the API key, wait a bit for the browser to process
+    if api_key and server.api_key:
         time.sleep(1.5)  # Give browser time to receive response and start redirect
 
     # Shutdown server in a separate thread to avoid deadlock
@@ -324,8 +376,8 @@ def authenticate_via_browser(auth_url="https://omnara.com"):
 
     server.server_close()
 
-    if server.api_key:
-        return server.api_key
+    if api_key:
+        return api_key
     else:
         raise Exception("Authentication failed - no API key received")
 
@@ -336,6 +388,11 @@ def ensure_api_key(args):
     if hasattr(args, "api_key") and args.api_key:
         return args.api_key
 
+    # Check if API key is in environment variable
+    env_api_key = os.environ.get("OMNARA_API_KEY")
+    if env_api_key:
+        return env_api_key
+
     # Try to load from storage
     api_key = load_stored_api_key()
     if api_key:
@@ -370,6 +427,9 @@ def cmd_headless(args, unknown_args):
     if hasattr(args, "base_url") and args.base_url:
         new_argv.extend(["--base-url", args.base_url])
 
+    if hasattr(args, "name") and args.name:
+        new_argv.extend(["--name", args.name])
+
     # Add headless-specific flags
     if hasattr(args, "prompt") and args.prompt:
         new_argv.extend(["--prompt", args.prompt])
@@ -428,9 +488,6 @@ def run_agent_chat(args, unknown_args):
             f"Unknown agent: {agent}. Supported agents: {', '.join(AGENT_CONFIGS.keys())}"
         )
 
-    # Dynamically import the appropriate wrapper
-    import importlib
-
     module = importlib.import_module(config["module"])
     wrapper_main = getattr(module, config["function"])
 
@@ -441,6 +498,10 @@ def run_agent_chat(args, unknown_args):
     if hasattr(args, "base_url") and args.base_url:
         new_argv.extend(["--base-url", args.base_url])
 
+    # Add name flag if provided
+    if hasattr(args, "name") and args.name:
+        new_argv.extend(["--name", args.name])
+
     # Add Claude-specific flags
     if hasattr(args, "permission_mode") and args.permission_mode:
         new_argv.extend(["--permission-mode", args.permission_mode])
@@ -451,6 +512,10 @@ def run_agent_chat(args, unknown_args):
     ):
         new_argv.append("--dangerously-skip-permissions")
 
+    # Add idle-delay flag if provided
+    if hasattr(args, "idle_delay") and args.idle_delay:
+        new_argv.extend(["--idle-delay", str(args.idle_delay)])
+
     # Add any additional arguments
     if unknown_args:
         new_argv.extend(unknown_args)
@@ -462,7 +527,7 @@ def run_agent_chat(args, unknown_args):
         sys.argv = original_argv
 
 
-def cmd_serve(args):
+def cmd_serve(args, unknown_args=None):
     """Handle the 'serve' subcommand"""
     # Run the webhook server with appropriate tunnel configuration
     cmd = [
@@ -471,13 +536,7 @@ def cmd_serve(args):
         "integrations.webhooks.claude_code.claude_code",
     ]
 
-    if args.skip_permissions:
-        cmd.append("--dangerously-skip-permissions")
-
-    if args.debug:
-        cmd.append("--debug")
-
-    # Handle tunnel configuration
+    # Handle tunnel configuration (webhook-specific)
     if not args.no_tunnel:
         # Default: use Cloudflare tunnel
         cmd.append("--cloudflare-tunnel")
@@ -489,7 +548,16 @@ def cmd_serve(args):
     if args.port is not None:
         cmd.extend(["--port", str(args.port)])
 
-    subprocess.run(cmd)
+    # Pass through ALL unknown arguments (including permission flags)
+    # These will flow through to HeadlessClaudeRunner and then to Claude CLI
+    if unknown_args:
+        cmd.extend(unknown_args)
+
+    try:
+        subprocess.run(cmd)
+    except KeyboardInterrupt:
+        print("\n[INFO] Webhook server stopped by user")
+        sys.exit(0)
 
 
 def cmd_mcp(args):
@@ -514,7 +582,11 @@ def cmd_mcp(args):
     if args.disable_tools:
         cmd.append("--disable-tools")
 
-    subprocess.run(cmd)
+    try:
+        subprocess.run(cmd)
+    except KeyboardInterrupt:
+        print("\n[INFO] MCP server stopped by user")
+        sys.exit(0)
 
 
 def add_global_arguments(parser):
@@ -551,6 +623,11 @@ def add_global_arguments(parser):
         default="claude",
         help="Which AI agent to use (default: claude)",
     )
+    parser.add_argument(
+        "--name",
+        default=None,
+        help="Name of the omnara agent (defaults to the name of the underlying agent)",
+    )
     parser.add_argument(
         "--permission-mode",
         choices=["acceptEdits", "bypassPermissions", "default", "plan"],
@@ -561,6 +638,12 @@ def add_global_arguments(parser):
         action="store_true",
         help="Bypass all permission checks. Recommended only for sandboxes with no internet access.",
     )
+    parser.add_argument(
+        "--idle-delay",
+        type=float,
+        default=1.0,
+        help="Delay in seconds before considering Claude idle (default: 1.0)",
+    )
 
 
 def main():
@@ -621,11 +704,8 @@ Examples:
     serve_parser.add_argument(
         "--port", type=int, help="Port to run the webhook server on (default: 6662)"
     )
-    serve_parser.add_argument(
-        "--skip-permissions",
-        action="store_true",
-        help="Skip permission prompts in Claude Code - USE WITH CAUTION",
-    )
+    # All permission-related args will be passed through as unknown_args
+    # No need to explicitly define them here
     serve_parser.add_argument(
         "--debug",
         action="store_true",
@@ -665,6 +745,20 @@ Examples:
         "headless",
         help="Run Claude Code in headless mode (controlled via web dashboard)",
     )
+    # Add the same global arguments to headless subcommand
+    headless_parser.add_argument(
+        "--api-key", help="API key for authentication (uses stored key if not provided)"
+    )
+    headless_parser.add_argument(
+        "--base-url",
+        default="https://agent-dashboard-mcp.onrender.com",
+        help="Base URL of the Omnara API server",
+    )
+    headless_parser.add_argument(
+        "--auth-url",
+        default="https://omnara.com",
+        help="Base URL of the Omnara frontend for authentication",
+    )
     headless_parser.add_argument(
         "--prompt",
         default="You are starting a coding session",
@@ -719,7 +813,7 @@ Examples:
 
     # Handle subcommands
     if args.command == "serve":
-        cmd_serve(args)
+        cmd_serve(args, unknown_args)
     elif args.command == "mcp":
         cmd_mcp(args)
     elif args.command == "headless":
